# coding: utf-8
import pyswf, datetime, math

def add_horizontal_wf(swf_trace, num_jobs, submit_time, target_uid, runtime = 1800, alloc_cpus = 96, requested_time = 2400, partition_id = 1, fs_runtime = 0):
    """
    :param fs_runtime: runtime of the first job added to the target_uid. This controls the fairshare value
    """
    if fs_runtime != 0:
        # correct all the submit time of the other jobs
        swf_trace.job_list.submit_time += fs_runtime
        # correct workflow submit time
        submit_time += fs_runtime

        swf_trace.add_job(0, int(fs_runtime), int(swf_trace.max_procs), int(target_uid), int(fs_runtime), gid = int(target_uid), partition_id = int(partition_id))
    
    for i in range(num_jobs):
        swf_trace.add_job(int(submit_time), int(runtime), int(alloc_cpus), int(target_uid), int(requested_time), gid = int(target_uid), partition_id = int(partition_id))
        
def add_wrapped_horizontal_wf(swf_trace, num_jobs, submit_time, target_uid, runtime = 1800, alloc_cpus = 96, requested_time = 2400, partition_id = 1, fs_runtime = 0):
    """
    :param fs_runtime: runtime of the first job added to the target_uid. This controls the fairshare value
    """
    if fs_runtime != 0:
        # correct all the submit time of the other jobs
        swf_trace.job_list.submit_time += fs_runtime
        # correct workflow submit time
        submit_time += fs_runtime

        swf_trace.add_job(0, int(fs_runtime), int(swf_trace.max_procs), int(target_uid), int(fs_runtime), gid = int(target_uid), partition_id = int(partition_id))
    
    swf_trace.add_job(int(submit_time), int(runtime), int(num_jobs*alloc_cpus), int(target_uid), int(requested_time), gid = int(target_uid), partition_id = int(partition_id))

def add_vertical_wf(swf_trace, num_jobs, submit_time, target_uid, runtime = 1800, alloc_cpus = 96, requested_time = 2400, partition_id = 1, fs_runtime = 0):
    """
    :param fs_runtime: runtime of the first job added to the target_uid. This controls the fairshare value
    """
    if fs_runtime != 0:
        # correct all the submit time of the other jobs
        swf_trace.job_list.submit_time += fs_runtime
        # correct workflow submit time
        submit_time += fs_runtime

        swf_trace.add_job(0, int(fs_runtime), int(swf_trace.max_procs), int(target_uid), int(fs_runtime), gid = int(target_uid), partition_id = int(partition_id))

    for i in range(num_jobs):
        if i > 0:
            preceding_job_id = swf_trace.job_list[swf_trace.job_list.uid == target_uid].job_id.iloc[-1]
        else:
            preceding_job_id = -1
        swf_trace.add_job(int(submit_time + i*runtime), int(runtime), int(alloc_cpus), int(target_uid), int(requested_time), gid = int(target_uid), partition_id = int(partition_id), preceding=int(preceding_job_id))

def add_wrapped_vertical_wf(swf_trace, num_jobs, submit_time, target_uid, runtime = 1800, alloc_cpus = 96, requested_time = 2400, partition_id = 1, fs_runtime = 0):
    """
    :param fs_runtime: runtime of the first job added to the target_uid. This controls the fairshare value
    """
    if fs_runtime != 0:
        # correct all the submit time of the other jobs
        swf_trace.job_list.submit_time += fs_runtime
        # correct workflow submit time
        submit_time += fs_runtime

        swf_trace.add_job(0, int(fs_runtime), int(swf_trace.max_procs), int(target_uid), int(fs_runtime), gid = int(target_uid), partition_id = int(partition_id))

    swf_trace.add_job(int(submit_time), int(num_jobs*runtime), alloc_cpus, int(target_uid), int(requested_time), gid = int(target_uid), partition_id = int(partition_id))


if __name__ == "__main__":

    curie = pyswf.SWFtrace.from_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/CEA-Curie/slices/curie_493_499_zeroed.swf")
    curie.max_procs = 93312
    curie.max_nodes = 5832

    # submit time

    d14_10 = datetime.datetime(year=2012, month=6, day=14, hour=10)
    d14_15 = datetime.datetime(year=2012, month=6, day=14, hour=15)
    d14_20 = datetime.datetime(year=2012, month=6, day=14, hour=20)
    d15_10 = datetime.datetime(year=2012, month=6, day=15, hour=10)
    d15_15 = datetime.datetime(year=2012, month=6, day=15, hour=15)
    d15_20 = datetime.datetime(year=2012, month=6, day=15, hour=20)
    s14_10 = (d14_10 - curie.start_time)
    s14_15 = (d14_15 - curie.start_time)
    s14_20 = (d14_20 - curie.start_time)
    s15_10 = (d15_10 - curie.start_time)
    s15_15 = (d15_15 - curie.start_time)
    s15_20 = (d15_20 - curie.start_time)

    # clean copy of the original job list

    clean_job_list = curie.job_list.copy()

    # vertical unwrapped

    add_vertical_wf(curie, 7, s14_10.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_vertical_14_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s14_15.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_vertical_14_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s14_20.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_vertical_14_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s15_10.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_vertical_15_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s15_15.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_vertical_15_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s15_20.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_vertical_15_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    # horizontal unwrapped

    add_horizontal_wf(curie, 12, s14_10.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_horizontal_14_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s14_15.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_horizontal_14_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s14_20.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_horizontal_14_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s15_10.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_horizontal_15_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s15_15.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_horizontal_15_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s15_20.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_cams240_horizontal_15_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    # vertical wrapped

    add_wrapped_vertical_wf(curie, 7, s14_10.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_vertical_14_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s14_15.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_vertical_14_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s14_20.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_vertical_14_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s15_10.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_vertical_15_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s15_15.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_vertical_15_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s15_20.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_vertical_15_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    # horizontal wrapped 

    add_wrapped_horizontal_wf(curie, 12, s14_10.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_horizontal_14_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s14_15.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_horizontal_14_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s14_20.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_horizontal_14_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s15_10.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_horizontal_15_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s15_15.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_horizontal_15_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s15_20.total_seconds(), 723)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/best/best_curie_493_499_wrapped_cams240_horizontal_15_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    # WORST FAIR SHARE

    # computes the largest usage in terms of runtime * cores for each of the users
    max_runtime_cores = 0
    for user in curie.job_list.uid.unique():
        user_jobs = curie.job_list[curie.job_list.uid == user]
        total_user_runtime = (user_jobs.runtime*user_jobs.alloc_cpus).sum()
        if total_user_runtime > max_runtime_cores:
            max_runtime_cores = total_user_runtime

    fs_runtime = math.ceil(max_runtime_cores/curie.max_procs)
    
    # vertical unwrapped

    add_vertical_wf(curie, 7, s14_10.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_vertical_14_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s14_15.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_vertical_14_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s14_20.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_vertical_14_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s15_10.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_vertical_15_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s15_15.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_vertical_15_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_vertical_wf(curie, 7, s15_20.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_vertical_15_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    # horizontal unwrapped

    add_horizontal_wf(curie, 12, s14_10.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_horizontal_14_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s14_15.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_horizontal_14_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s14_20.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_horizontal_14_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s15_10.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_horizontal_15_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s15_15.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_horizontal_15_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_horizontal_wf(curie, 12, s15_20.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_cams240_horizontal_15_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    # vertical wrapped

    add_wrapped_vertical_wf(curie, 7, s14_10.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_vertical_14_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s14_15.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_vertical_14_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s14_20.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_vertical_14_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s15_10.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_vertical_15_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s15_15.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_vertical_15_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_vertical_wf(curie, 7, s15_20.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_vertical_15_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    # horizontal wrapped 

    add_wrapped_horizontal_wf(curie, 12, s14_10.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_horizontal_14_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s14_15.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_horizontal_14_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s14_20.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_horizontal_14_6_12_20.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s15_10.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_horizontal_15_6_12_10.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s15_15.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_horizontal_15_6_12_15.swf")
    curie.job_list = clean_job_list.copy()

    add_wrapped_horizontal_wf(curie, 12, s15_20.total_seconds(), 723, fs_runtime = fs_runtime)
    curie.to_swf("/home/mgimenez/Documents/local/wrapper paper/workloads/with workflow/curie/week 493-499/thursday friday/worst/worst_curie_493_499_wrapped_cams240_horizontal_15_6_12_20.swf")
    curie.job_list = clean_job_list.copy()
